<?php
/**
 * NCAAFEvent is a concrete implementation of {@see MythEvent}. 
 *
 * @version $Id: NCAAFEvent.php,v 1.3.2.1 2006/11/05 23:06:57 dbattams Exp $
 * @author Michael Lynch <lynchmv@gmail.com>
 * @package MRE
 * @copyright 2006 GPL v2
 */

/**#@+
 * Required include
 */
require_once(APP_ROOT . '/class/MythEvent.php');
/**#@-*/

/**
 * NCAAFEvent monitors NCAAF games via ESPN.com
 *
 * NCAAFEvent will monitor the status of NCAAF games via the information provided
 * by ESPN.com
 *
 * @package MRE
 */
class NCAAFEvent extends MythEvent
{
   /**
    * URL to grab game info from
    */
   const url = 'http://scores.espn.go.com/ncf/scoreboard';

   /**
    * Find and read a more detailed scoreboard page for game data 
    *
    * Given the contents of a scoreboard page, find the link named, $name, and
    * read the scoreboard data for that name.  If successful, mod the caller's
    * data to use the newly read data.
    * @param string The name of the scoreboard to read
    * @param array Generic scoreboard data as read from the file() function
    * @return void
    */
   static protected function findFullScoreboard($name, &$data)
   {
      if(!strlen($name))
         return;

      $conf = SysConf::getInstance();
      if($conf->getOpt('verbose') >= SysConf::DEBUG)
         echo "Attempting to change to source: $name\n";
      foreach($data as $line)
      {
         if(preg_match('/\\<option value="([^"]+)"\>' . preg_quote($name) . '\<\/option\>/', $line, $match))
         {
            $fullData = file(self::url . $match[1]);
            if($fullData !== FALSE)
               $data = $fullData;
            else if($conf->getOpt('verbose') >= SysConf::DEBUG)
               echo "Game data update failed!\n";
            break;
         }
      }
      return;
   }

   /**
    * Class constructor.
    *
    * @param string A string describing the game to be monitored
    */
   public function __construct($gameInfo)
   {
      parent::__construct();

      $criteria = $this->generateSearchTerms($gameInfo);
      if(!sizeof($criteria))
         trigger_error("Unable to generate valid search terms! [$gameInfo]", E_USER_WARNING);
      else
      {
         $sources = array('', 'All Div I-A', 'All Div I-AA');
         $data = file(self::url);
         if($data !== FALSE)
         {
            foreach($sources as $source)
            {
               $games = array();
               self::findFullScoreboard($source, $data);
               // Split the huge string into individual game data
               foreach($data as $line)
                  if(preg_match('/\<td class="teamTop.*".*\>.*/', $line))
                     $games[] = $line;
               if(!sizeof($games))
                  trigger_error("Unable to find valid game data within source!", E_USER_WARNING);
               foreach($criteria as $term)
               {
                  foreach($games as $game)
                  {
                     if(preg_match('/\<a.*\>[^A-Za-z]*' . preg_quote($term) . '\<\/a\>/', $game))
                        if(preg_match('/teamTop.*\>(.*)\<\/td\>/U', $game, $matches))
                        {
                           $this->setStatus($matches[1]);
                           if(preg_match('/^(Final|Postponed)/', $matches[1]))
                              $this->setEventOver(TRUE);
                           else
                              $this->setEventOver(FALSE);
                           $this->setValid(TRUE);
                           break 3;
                        }
                  }
               }
            }
         }
      }
      if(!$this->isValid())
         trigger_error("Unable to find NCAAF info for '$gameInfo'!  The NCAAF parser may need to be updated.", E_USER_WARNING);
   }

   /**
    * Override of parent implementation
    *
    * This override is much more strict about finding game information.  Why?
    * We just can't trust a dat file for the NCAA, it's always going to be
    * incomplete.  So I'd rather be very strict about finding a matching game
    * and take a chance of no event monitor being attached than to attach a
    * monitor for the wrong game to a recording and have someone miss the end
    * of a game on account of an MRE mistake.
    *
    * @param string The description of the game we're monitoring
    * @return array Valid seach criteria for monitoring this game
    */ 
   public function generateSearchTerms($gameInfo)
   {
      if(preg_match('/\s*(.*) (?:at|vs\.{0,1}) (.*)$/', $gameInfo, $match))
         return array($match[1], $match[2]);
      return array();
   }  
}

?>
